@@ -1,5 +1,5 @@
 /***************************************************************************
- * RT2400 SourceForge Project - http://rt2400.sourceforge.net              *
+ * RT2x00 SourceForge Project - http://rt2x00.serialmonkey.com              *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,35 +17,38 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  *                                                                         *
  *   Licensed under the GNU GPL                                            *
- *   Original code supplied under license from RaLink Inc, 2003.           *
+ *   Original code supplied under license from RaLink Inc, 2004.           *
  ***************************************************************************/
 
- /***************************************************************************
- *      Module Name: rtmp_def.h
+/***************************************************************************
+ *	Module Name:	rtmp_def.h
  *
- *      Abstract: Miniport related definition header
+ *	Abstract:
  *
- *      Revision History:
- *      Who             When            What
- *      --------        -----------     -----------------------------
- *      PaulL           1st  Oct 02     Created
- *      MarkW           9th  Feb 04     Baseline of code
- *      RobinC          21st Dec 04     RFMON Support
- *      MarkW           21st Dec 04     iwconfig mode monitor support
+ *	Revision History:
+ *	Who		When		What
+ *	--------	----------	-------------------------------
+ *	Name		Date		Modification logs
+ *	Jan Lee		2005-06-01	Release
  ***************************************************************************/
 
-
 #ifndef __RTMP_DEF_H__
 #define __RTMP_DEF_H__
 
 //
 //  Debug information verbosity: lower values indicate higher urgency
 //
-#define RT_DEBUG_ERROR      KERN_ERR
-#define RT_DEBUG_WARN       KERN_WARNING
-#define RT_DEBUG_TRACE      KERN_NOTICE
-#define RT_DEBUG_INFO       KERN_INFO
-#define RT_DEBUG_LOUD       KERN_DEBUG
+#define RT_DEBUG_OFF        0
+#define RT_DEBUG_ERROR      1
+#define RT_DEBUG_WARN       2
+#define RT_DEBUG_TRACE      4
+#define RT_DEBUG_INFO       8
+#define RT_DEBUG_LOUD       16
+#define RT_DEBUG_CNTL       RT_DEBUG_ERROR
+#define RT_DEBUG_ADHOC      RT_DEBUG_TRACE
+
+#define NIC_TAG             ((ULONG)'0752')
+#define NIC_DBG_STRING      "RTUSB  "
 
 //
 // update the driver version number every time you release a new driver
@@ -53,6 +56,16 @@
 //
 #define NIC_VENDOR_DRIVER_VERSION   0x00010001
 
+//
+// NDIS version in use by the NIC driver.
+// The high byte is the major version. The low byte is the minor version.
+//
+#ifdef  NDIS51_MINIPORT
+#define NIC_DRIVER_VERSION      0x0501
+#else
+#define NIC_DRIVER_VERSION      0x0500
+#endif
+
 //
 // NDIS media type, current is ethernet, change if native wireless supported
 //
@@ -61,56 +74,28 @@
 #define NIC_MAX_PACKET_SIZE     2304
 #define NIC_HEADER_SIZE         14
 
-#define DEBUG_TASK_DELAY        (2 * HZ)        // Delay for miniport debug task timer, 2 sec.
-//
-// NIC PCI Device and vendor IDs
-// Use D-Link for now
-//
-#define NIC_PCI_DEVICE_ID       0x0101
-#define NIC_PCI_VENDOR_ID       0x1814
 
 
-#define MAX_SEND_SKB_COUNT		16
-
-//
-// interface type, we use PCI
-//
-#define NIC_INTERFACE_TYPE      NdisInterfacePci
-#define NIC_INTERRUPT_MODE      NdisInterruptLevelSensitive
-
-//
-// buffer size passed in NdisMQueryAdapterResources
-// We should only need three adapter resources (IO, interrupt and memory),
-// Some devices get extra resources, so have room for 10 resources
-//
-#define NIC_RESOURCE_BUF_SIZE   (sizeof(NDIS_RESOURCE_LIST) + (10*sizeof(CM_PARTIAL_RESOURCE_DESCRIPTOR)))
-
-//
-// IO space length
-//
-#define NIC_MAP_IOSPACE_LENGTH  sizeof(CSR_STRUC)
-
-//
-// NIC Cmpany description
-//
-#define NIC_VENDOR_DESC         "Ralink Technology Inc."
-
 //
 // Entry number for each DMA descriptor ring
 //
+//blue,#define TX_RING_SIZE            16
 #define TX_RING_SIZE            32
-#define ATIM_RING_SIZE          4
-#define PRIO_RING_SIZE          16    // 8
-#define RX_RING_SIZE            32
-#define	BEACON_RING_SIZE		1
-#define DESCRIPTOR_REQUIRED     ((TX_RING_SIZE) + (ATIM_RING_SIZE) + (PRIO_RING_SIZE) + (RX_RING_SIZE) + (BEACON_RING_SIZE))
+//blue,#define ATIM_RING_SIZE          4
+#define PRIO_RING_SIZE          32
+//#define RX_RING_SIZE            1
+#define RX_RING_SIZE            8
+#define	BEACON_RING_SIZE		2
+//blue,#define DESCRIPTOR_REQUIRED     ((TX_RING_SIZE) + (ATIM_RING_SIZE) + (PRIO_RING_SIZE) + (RX_RING_SIZE) + (BEACON_RING_SIZE))
 #define	MGMT_RING_SIZE			32
-#define RING_DESCRIPTOR_SIZE    32
-#define TX_BUFFER_SIZE          2048
+//blue,#define RING_DESCRIPTOR_SIZE    48
+//steven:for test#define TX_BUFFER_SIZE          2048
 #define PRIO_BUFFER_SIZE        1024  // 2048
-#define RX_BUFFER_SIZE          2048
-#define ATIM_BUFFER_SIZE        2048
-#define BEACON_BUFFER_SIZE      2048
+//steven:for test#define RX_BUFFER_SIZE          2064	//2048
+//blue,#define BUFFER_SIZE				2064	//2048
+#define BUFFER_SIZE				2400	//2048
+//blue,#define ATIM_BUFFER_SIZE        2048
+//blue,#define BEACON_BUFFER_SIZE      2048
 #define	MAX_FRAME_SIZE			2346					// Maximum 802.11 frame size
 #define ALLOC_RX_PACKET_POOL    (RX_RING_SIZE)
 #define ALLOC_RX_BUFFER_POOL    (ALLOC_RX_PACKET_POOL)
@@ -118,14 +103,18 @@
 #define	ATIM_RING				0xb
 #define	PRIO_RING				0xc
 #define	RX_RING					0xd
+#define	BEACON_RING				0xe
+#define	NULL_RING				0xf
 #define	MAX_TX_PROCESS			2
 #define	MAX_RX_PROCESS			4
 #define	MAX_CLIENT				4
 #define	MAX_MCAST_LIST_SIZE		32
 
 //  RTMP_ADAPTER flags
-#define fRTMP_ADAPTER_MAP_REGISTER          0x00000001
-#define fRTMP_ADAPTER_INTERRUPT_IN_USE      0x00000002
+//#define fRTMP_ADAPTER_MAP_REGISTER          0x00000001
+//#define fRTMP_ADAPTER_INTERRUPT_IN_USE      0x00000002
+#define	fRTMP_ADAPTER_TEST_MODE				0x00000001//blue
+#define fRTMP_ADAPTER_MLME_RESET_IN_PROGRESS     0x00000002 //blue
 #define fRTMP_ADAPTER_HARDWARE_ERROR        0x00000004
 #define fRTMP_ADAPTER_SEND_PACKET_ERROR     0x00000010
 #define fRTMP_ADAPTER_RECEIVE_PACKET_ERROR  0x00000020
@@ -136,11 +125,21 @@
 #define fRTMP_ADAPTER_ATIM_RING_ALLOCATED   0x00000400
 #define fRTMP_ADAPTER_PRIO_RING_ALLOCATED   0x00000800
 #define fRTMP_ADAPTER_RX_RING_ALLOCATED     0x00001000
-#define fRTMP_ADAPTER_INTERRUPT_ACTIVE      0x00002000
+#define fRTMP_ADAPTER_BSS_JOIN_IN_PROGRESS  0x00002000
 #define fRTMP_ADAPTER_BSS_SCAN_IN_PROGRESS  0x00004000
-#define	fRTMP_ADAPTER_MEDIA_STATE_CHANGE	0x00008000
+#define	fRTMP_ADAPTER_REASSOC_IN_PROGRESS	0x00008000
 #define	fRTMP_ADAPTER_MEDIA_STATE_PENDING	0x00010000
 #define	fRTMP_ADAPTER_RADIO_OFF				0x00020000
+#define	fRTMP_ADAPTER_BULKOUT_RESET			0x00100000
+#define	fRTMP_ADAPTER_BULKIN_RESET			0x00200000
+#define fRTMP_ADAPTER_RESET_DATA_SW_QUEUE   0x00400000
+#define fRTMP_ADAPTER_RESET_PIPE_IN_PROGRESS    0x00800000
+#define fRTMP_ADAPTER_SCAN_CHANNEL_IN_PROGRESS 0x01000000
+#ifdef RALINK_ATE
+#define	fRTMP_ADAPTER_ATE_START_TX			0x01000000
+#define	fRTMP_ADAPTER_ATE_START_TX_CARRIER	0x02000000
+#define	fRTMP_ADAPTER_ATE_START_RX			0x04000000
+#endif
 
 // Lock bit for accessing different ring buffers
 #define fRTMP_ADAPTER_TX_RING_BUSY      	0x80000000
@@ -153,9 +152,20 @@
 #define	fRTMP_ADAPTER_PRIO_QUEUE_BUSY      	0x04000000
 
 //
-//  STA operation status flags
+//	STA operation status flags
 //
-#define fOP_STATUS_BSSID_SET				0x00008000
+#define fOP_STATUS_BSSID_SET				0x00004000
+
+//
+// Flags for Bulkflags control for bulk out data
+//
+#define	fRTUSB_BULK_OUT_DATA_NULL			0x00000001
+#define	fRTUSB_BULK_OUT_DATA_NORMAL			0x00000002
+#define	fRTUSB_BULK_OUT_MLME				0x00000004
+#define	fRTUSB_BULK_OUT_BEACON_1			0x00000008
+#define	fRTUSB_BULK_OUT_BEACON_0			0x00000010
+#define	fRTUSB_BULK_OUT_PSPOLL				0x00000020
+#define	fRTUSB_BULK_OUT_DATA_FRAG			0x00000040
 
 //
 // Error code section
@@ -202,19 +212,24 @@
 #define CRC_LEN                           4
 #define MAC_ADDR_LEN                      6
 #define TIMESTAMP_LEN                     8
-#define MAX_LEN_OF_SUPPORTED_RATES        8
+#define MAX_LEN_OF_SUPPORTED_RATES        12    // 1, 2, 5.5, 11, 6, 9, 12, 18, 24, 36, 48, 54
 #define MAX_NUM_OF_POWER_LEVEL            8
 #define MAX_NUM_OF_DOMAIN                 8
-#define MAX_LEN_OF_KEY                    13      // 13 octets == 104 bits
-#define MAX_LEN_OF_CHANNELS               14      // 11 or 14 channels
+#define MAX_LEN_OF_KEY                    32      // 32 octets == 256 bits, Redefine for WPA
+#define MAX_LEN_OF_CHANNELS               42      // 14 channels @2.4G +  12@UNII + 4 @MMAC + 11 @HiperLAN2 + 1 as NULL termination
 #define MAX_LEN_OF_PEER_KEY               16
 #define MAC_HDR_LEN                       24
+#define MAX_LEN_OF_MANUFACTURE_ID         32
+#define MAX_LEN_OF_PRODUCT_ID             32
 #define MAX_LEN_OF_MAC_TABLE              32
 #define MAX_LEN_OF_SSID                   32
 #define CIPHER_TEXT_LEN                   128
 #define HASH_TABLE_SIZE                   256
 #define MAX_LEN_OF_MLME_BUFFER            512
 #define MAX_FRAME_LEN                     2338
+#define	MAX_VIE_LEN                       128	// New for WPA cipher suite variable IE sizes.
+#define MAX_MLME_HANDLER_MEMORY           20    //each them cantains  MAX_LEN_OF_MLME_BUFFER size
+#define MAX_INI_BUFFER_SIZE               4096
 
 #define MAX_TX_POWER_LEVEL                100   /* mW */
 #define MAX_RSSI_TRIGGER                 -10    /* dBm */
@@ -226,6 +241,9 @@
 // key related definitions
 #define SHARE_KEY_NO                      4
 #define MAX_LEN_OF_SHARE_KEY              16
+#define PAIRWISE_KEY_NO                   4
+#define GROUP_KEY_NO                      4
+#define	PMKID_NO							4	// Number of PMKID saved supported
 
 // power status related definitions
 #define PWR_ACTIVE                        0
@@ -237,7 +255,7 @@
 #define ASSOC_MODE                        0x20
 
 #define AUTH_MODE_OPEN                    0x00
-#define AUTH_MODE_KEY                     0x01
+#define AUTH_MODE_SHARED                  0x01
 #define AUTH_MODE_AUTO_SWITCH             0x03
 #define AUTH_MODE_DEAUTH                  0x04
 #define AUTH_MODE_UPLAYER                 0x05 // reserved for 802.11i use
@@ -249,7 +267,8 @@
 #define BSS_INDEP                         0  // = Ndis802_11IBSS
 #define BSS_INFRA                         1  // = Ndis802_11Infrastructure
 #define BSS_ANY                           2  // = Ndis802_11AutoUnknown
-#define BSS_MONITOR                       3  // = Ndis802_11Monitor
+#define BSS_MONITOR                       3  // = RMFON MODE
+
 // #define BSS_UNKNOWN                       0xff
 
 
@@ -276,6 +295,10 @@
 #define REASON_CLS3ERR                    7
 #define REASON_DISASSOC_STA_LEAVING       8
 #define REASON_STA_REQ_ASSOC_NOT_AUTH     9
+#define	REASON_INVALID_IE                 13
+#define	REASON_MIC_FAILURE                14
+#define	REASON_4_WAY_HANDSHAKE_TIMEOUT    15
+#define	REASON_GROUP_KEY_UPDATE_TIMEOUT   16
 
 // Status code definitions
 #define MLME_SUCCESS                      0
@@ -307,7 +330,23 @@
 #define IE_CF_PARM                        4
 #define IE_TIM                            5
 #define IE_IBSS_PARM                      6
+#define IE_COUNTRY                        7     // 802.11d
+#define IE_802_11D_REQUEST                10    // 802.11d
 #define IE_CHALLENGE_TEXT                 16
+#define IE_POWER_CONSTRAINT               32    // 802.11h d3.3
+#define IE_POWER_CAPABILITY               33    // 802.11h d3.3
+#define IE_TPC_REQUEST                    34    // 802.11h d3.3
+#define IE_TPC_REPORT                     35    // 802.11h d3.3
+#define IE_SUPP_CHANNELS                  36    // 802.11h d3.3
+#define IE_CHANNEL_SWITCH_ANNOUNCEMENT    37    // 802.11h d3.3
+#define IE_MEASUREMENT_REQUEST            38    // 802.11h d3.3
+#define IE_MEASUREMENT_REPORT             39    // 802.11h d3.3
+#define IE_QUIET                          40    // 802.11h d3.3
+#define IE_IBSS_DFS                       41    // 802.11h d3.3
+#define IE_ERP                            42    // 802.11g
+#define IE_EXT_SUPP_RATES                 50    // 802.11g
+#define IE_WPA                            221   // WPA
+#define IE_RSN                            48    // 802.11i d3.0
 
 #define CNTL_FUNC_SIZE                    1
 
@@ -425,8 +464,11 @@
 #define SUBTYPE_DISASSOC        10
 #define SUBTYPE_AUTH            11
 #define SUBTYPE_DEAUTH          12
+#define SUBTYPE_ACTION          13
 
 // Control Frame
+#define SUBTYPE_BLOCK_ACK_REQ   8
+#define SUBTYPE_BLOCK_ACK       9
 #define SUBTYPE_PS_POLL         10
 #define SUBTYPE_RTS             11  // 1011
 #define SUBTYPE_CTS             12  // 1100
@@ -443,12 +485,21 @@
 #define SUBTYPE_CFACK               5  // 0101
 #define SUBTYPE_CFPOLL              6
 #define SUBTYPE_CFACK_CFPOLL        7
+#define SUBTYPE_QDATA               8
+#define SUBTYPE_QDATA_CFACK         9
+#define SUBTYPE_QDATA_CFPOLL        10
+#define SUBTYPE_QDATA_CFACK_CFPOLL  11
+#define SUBTYPE_QOS_NULL            12
+#define SUBTYPE_QOS_CFACK           13
+#define SUBTYPE_QOS_CFPOLL          14
+#define SUBTYPE_QOS_CFACK_CFPOLL    15
 
 #define ASSOC_STATE_MACHINE        1
 #define AUTH_STATE_MACHINE         2
 #define AUTH_RSP_STATE_MACHINE     3
 #define SYNC_STATE_MACHINE         4
 #define MLME_CNTL_STATE_MACHINE    5
+#define WPA_PSK_STATE_MACHINE       6 //red, add for WPA-PSK
 
 //
 // rtmp_data.c use these definition
@@ -460,7 +511,8 @@
 #define	LENGTH_802_3			14
 #define LENGTH_802_3_TYPE		2
 #define LENGTH_802_1_H			8
-#define	LENGTH_CRC			4
+#define LENGTH_EAPOL_H          	4
+#define	LENGTH_CRC				4
 #define	MAX_SEQ_NUMBER			0x0fff
 
 #define	SUCCESS_WITHOUT_RETRY	0
@@ -473,6 +525,18 @@
 #define	RATE_2					1
 #define	RATE_5_5				2
 #define	RATE_11					3
+#define RATE_6                  4   // OFDM
+#define RATE_9                  5   // OFDM
+#define RATE_12                 6   // OFDM
+#define RATE_18                 7   // OFDM
+#define RATE_24                 8   // OFDM
+#define RATE_36                 9   // OFDM
+#define RATE_48                 10  // OFDM
+#define RATE_54                 11  // OFDM
+#define RATE_72                 12
+#define RATE_100                13
+#define RATE_FIRST_OFDM_RATE    RATE_6
+#define RATE_AUTO_SWITCH        255 // for PortCfg.FixedTxRate only
 
 #define	IFS_BACKOFF				0
 #define	IFS_SIFS				1
@@ -483,6 +547,7 @@
 #define	SHORT_RETRY				0
 
 // Country Region definition
+#define REGION_MIN              0
 #define	REGION_FCC				0		// 1-11
 #define	REGION_IC				1		// 1-11
 #define	REGION_ETSI				2		// 1-13
@@ -491,34 +556,47 @@
 #define	REGION_MKK				5		// 14
 #define	REGION_MKK1				6		// 1-14
 #define	REGION_ISRAEL			7		// 3-9
+#define REGION_MAX              REGION_ISRAEL
+
+#define	CIPHER_NONE				0
+#define	CIPHER_WEP64			1
+#define	CIPHER_WEP128			2
+#define	CIPHER_TKIP				3
+#define	CIPHER_AES				4
 
 // Stall execution time for ndisdpracquires[inlock in miniportReset function
 #define	WAIT_TIME_FOR_SPINLOCK	10		// usec
 
-// value domain for pAdapter->PortCfg.RfType and E2PROM
-#define RFIC_2420               0
-#define RFIC_2421               1
+// value domain for pAdapter->PortCfg.RfType
+#define RFIC_2522               0
+#define RFIC_2523               1
+#define RFIC_2524               2
+#define RFIC_2525               3
+#if 0//steven:modified by brand,blue
+#define RFIC_2525E              4
+#else
+#define RFIC_2525E              5
+#endif
+#define RFIC_5222               16
+// This chip is same as RT2526, it's for 11b only purpose
+#define	RFIC_2426				6
 
 // value domain for pAdapter->PortCfg.LedMode and E2PROM
 #define LED_MODE_DEFAULT        0
 #define LED_MODE_TXRX_ACTIVITY  1
 #define	LED_MODE_SINGLE			2		// Single LED mode, driver lid the LED as soon as driver up & enable tx activity right away
 #define	LED_MODE_ASUS			3		// Two LED modes, bit 16 acts as LED_MODE_SINGLE, bit 17 acts as RADIO status.
+#define	LED_MODE_ALPHA			4		// Two LED modes, bit 16 acts as LED_MODE_SINGLE, bit 17 acts as RADIO status.
 
-//
-// Ndis Packet Filter Bits (OID_GEN_CURRENT_PACKET_FILTER).
-//
-#define NDIS_PACKET_TYPE_DIRECTED               0x00000001
-#define NDIS_PACKET_TYPE_MULTICAST              0x00000002
-#define NDIS_PACKET_TYPE_ALL_MULTICAST          0x00000004
-#define NDIS_PACKET_TYPE_BROADCAST              0x00000008
-#define NDIS_PACKET_TYPE_SOURCE_ROUTING         0x00000010
-#define NDIS_PACKET_TYPE_PROMISCUOUS            0x00000020
-#define NDIS_PACKET_TYPE_SMT                    0x00000040
-#define NDIS_PACKET_TYPE_ALL_LOCAL              0x00000080
-#define NDIS_PACKET_TYPE_GROUP                  0x00001000
-#define NDIS_PACKET_TYPE_ALL_FUNCTIONAL         0x00002000
-#define NDIS_PACKET_TYPE_FUNCTIONAL             0x00004000
-#define NDIS_PACKET_TYPE_MAC_FRAME              0x00008000
+// RC4 init value, used fro WEP & TKIP
+#define PPPINITFCS32  0xffffffff   /* Initial FCS value */
 
+// 802.1X controlled port definition
+#define	WPA_802_1X_PORT_SECURED			1
+#define	WPA_802_1X_PORT_NOT_SECURED		2
+
+#define	PAIRWISE_KEY			1
+#define	GROUP_KEY				2
+
+#define MAX_PACKETS_IN_QUEUE        100
 #endif  // __RTMP_DEF_H__
