@@ -1,5 +1,5 @@
 /***************************************************************************
- * RT2400 SourceForge Project - http://rt2400.sourceforge.net              *
+ * RT2x00 SourceForge Project - http://rt2x00.serialmonkey.com              *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,23 +17,28 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  *                                                                         *
  *   Licensed under the GNU GPL                                            *
- *   Original code supplied under license from RaLink Inc, 2003.           *
+ *   Original code supplied under license from RaLink Inc, 2004.           *
  ***************************************************************************/
 
- /***************************************************************************
- *      Module Name: sanity.c
+/***************************************************************************
+ *	Module Name:	sanityc
  *
- *      Abstract:
+ *	Abstract:
  *
- *      Revision History:
- *      Who             When            What
- *      --------        -----------     -----------------------------
- *      MarkW           9th  Feb 04     Baseline of code
+ *	Revision History:
+ *	Who		When		What
+ *	--------	----------	-------------------------------
+ *	Name		Date		Modification logs
+ *	Jan Lee		2005-06-01	Release
  ***************************************************************************/
 
-
 #include "rt_config.h"
 
+UCHAR	WPA_OUI[] = {0x00, 0x50, 0xf2, 0x01};
+UCHAR	RSN_OUI[] = {0x00, 0x0f, 0xac};
+
+extern 	UCHAR	BCAST[];
+
 /*
     ==========================================================================
     Description:
@@ -42,8 +47,8 @@
         TRUE if all parameters are OK, FALSE otherwise
     ==========================================================================
  */
-BOOLEAN ScanReqParmSanity(
-    IN PRTMP_ADAPTER pAd,
+BOOLEAN MlmeScanReqSanity(
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT UCHAR *BssType,
@@ -64,7 +69,7 @@ BOOLEAN ScanReqParmSanity(
         return TRUE;
     else
     {
-        DBGPRINT(RT_DEBUG_TRACE, "ScanReqParmSanity fail - wrong BssType or ScanType\n");
+        DBGPRINT(RT_DEBUG_TRACE, "MlmeScanReqSanity fail - wrong BssType or ScanType\n");
         return FALSE;
     }
 }
@@ -77,8 +82,8 @@ BOOLEAN ScanReqParmSanity(
         TRUE if all parameters are OK, FALSE otherwise
     ==========================================================================
  */
-BOOLEAN StartParmSanity(
-    IN PRTMP_ADAPTER pAd,
+BOOLEAN MlmeStartReqSanity(
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT CHAR Ssid[],
@@ -90,7 +95,7 @@ BOOLEAN StartParmSanity(
 
     if (Info->SsidLen > MAX_LEN_OF_SSID)
     {
-        DBGPRINT(RT_DEBUG_TRACE, "StartParmSanity fail - wrong SSID length\n");
+        DBGPRINT(RT_DEBUG_TRACE, "MlmeStartReqSanity fail - wrong SSID length\n");
         return FALSE;
     }
 
@@ -106,10 +111,13 @@ BOOLEAN StartParmSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN MlmeAssocReqSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *ApAddr,
@@ -136,8 +144,8 @@ BOOLEAN MlmeAssocReqSanity(
         TRUE if all parameters are OK, FALSE otherwise
     ==========================================================================
  */
-BOOLEAN AuthReqParmSanity(
-    IN PRTMP_ADAPTER pAd,
+BOOLEAN MlmeAuthReqSanity(
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr,
@@ -151,13 +159,13 @@ BOOLEAN AuthReqParmSanity(
     *Timeout = Info->Timeout;
     *Alg = Info->Alg;
 
-    if ((*Alg == AUTH_MODE_KEY || *Alg == AUTH_MODE_OPEN) && !MAC_ADDR_IS_GROUP(*Addr))
+    if ((*Alg == Ndis802_11AuthModeShared || *Alg == Ndis802_11AuthModeOpen) && !MAC_ADDR_IS_GROUP(*Addr))
     {
         return TRUE;
     }
     else
     {
-        DBGPRINT(RT_DEBUG_TRACE, "AuthReqParmSanity fail - wrong algorithm\n");
+        DBGPRINT(RT_DEBUG_TRACE, "MlmeAuthReqSanity fail - wrong algorithm\n");
         return FALSE;
     }
 }
@@ -168,45 +176,13 @@ BOOLEAN AuthReqParmSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
-#if 0
-BOOLEAN JoinParmSanity(
-    IN PRTMP_ADAPTER pAd,
-    IN VOID *Msg,
-    IN ULONG MsgLen,
-    OUT ULONG *BssIdx,
-    OUT UCHAR SupportedRates[],
-    OUT UCHAR *SupportedRatesLen)
-{
-
-    PBSS_ENTRY   pBss;
-    MLME_JOIN_REQ_STRUCT *Info = (MLME_JOIN_REQ_STRUCT *)Msg;
-
-    *BssIdx = Info->BssIdx;
-    pBss = &pAd->Mlme.CntlAux.SsidBssTab.BssEntry[Info->BssIdx];
-
-    *SupportedRatesLen = pBss->RatesLen;
-    if (*SupportedRatesLen > MAX_LEN_OF_SUPPORTED_RATES)
-    {
-        DBGPRINT(RT_DEBUG_TRACE, ("JoinParmSanity fail - wrong SupportedRates IE\n"));
-        return FALSE;
-    }
-
-    memcpy(SupportedRates, pBss->Rates, pBss->RatesLen);
-    return TRUE;
-}
-#endif
-/*
-    ==========================================================================
-    Description:
-        MLME message sanity check
-    Return:
-        TRUE if all parameters are OK, FALSE otherwise
-    ==========================================================================
- */
-BOOLEAN PeerAssocParmSanity(
-    IN PRTMP_ADAPTER pAd,
+BOOLEAN PeerAssocRspSanity(
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr2,
@@ -214,44 +190,66 @@ BOOLEAN PeerAssocParmSanity(
     OUT USHORT *Status,
     OUT USHORT *Aid,
     OUT UCHAR Rates[],
-    OUT UCHAR *RatesLen)
+    OUT UCHAR *RatesLen,
+    OUT BOOLEAN *ExtendedRateIeExist)
 {
     CHAR          IeType, *Ptr;
     MACFRAME     *Fr = (MACFRAME *)Msg;
+    PBEACON_EID_STRUCT	eid_ptr;
 
     COPY_MAC_ADDR(Addr2, &Fr->Hdr.Addr2);
     Ptr = Fr->Octet;
 
-    memcpy(CapabilityInfo, &Fr->Octet[0], 2);
-    memcpy(Status,         &Fr->Octet[2], 2);
+	*CapabilityInfo = le16_to_cpup((PUSHORT)(&Fr->Octet[0]));
+	*Status = le16_to_cpup((PUSHORT)(&Fr->Octet[2]));
+	// Mask out unnecessary capability information
+	*CapabilityInfo &= SUPPORTED_CAPABILITY_INFO;
 
     if (*Status == MLME_SUCCESS)
     {
-        memcpy(Aid, &Fr->Octet[4], 2);
+		*Aid = le16_to_cpup((PUSHORT)(&Fr->Octet[4]));
         *Aid = (*Aid) & 0x3fff; // AID is low 14-bit
 
         // -- get supported rates from payload and advance the pointer
         IeType = Fr->Octet[6];
         *RatesLen = Fr->Octet[7];
-        if (IeType != IE_SUPP_RATES)
+        if ((IeType != IE_SUPP_RATES) || (*RatesLen > MAX_LEN_OF_SUPPORTED_RATES))
         {
-            DBGPRINT(RT_DEBUG_TRACE, "PeerAssocParmSanity fail - wrong SupportedRates IE\n");
+            DBGPRINT(RT_DEBUG_TRACE, "PeerAssocRspSanity fail - wrong SupportedRates IE\n");
             return FALSE;
         }
         else
-        {
-            int		index;
-            UCHAR	tmp_rate, i;
+       		memcpy(Rates, &Fr->Octet[8], *RatesLen);
 
-            i = 0;
-			for (index = 0; index < *RatesLen; index++)
-           	{
-            	// Mask out basic rate set bit
-            	tmp_rate = Fr->Octet[8 + index] & 0x7f;
-            	if ((tmp_rate == 0x02) || (tmp_rate == 0x04) || (tmp_rate == 0x0B) || (tmp_rate == 0x16))
-            		Rates[i++] = Fr->Octet[8 + index];	// Save rate with basic rate set bit if exists
+        // many AP implement proprietary IEs in non-standard order, we'd better
+        // tolerate mis-ordered IEs to get best compatibility
+        *ExtendedRateIeExist = FALSE;
+        eid_ptr = (PBEACON_EID_STRUCT) &Fr->Octet[8 + (*RatesLen)];
+
+        // get variable fields from payload and advance the pointer
+        while (((UCHAR*)eid_ptr + eid_ptr->Len + 1) < ((UCHAR*)Fr + MsgLen))
+        {
+            switch (eid_ptr->Eid)
+            {
+                case IE_EXT_SUPP_RATES:
+                    *ExtendedRateIeExist = TRUE;
+                    if ((*RatesLen + eid_ptr->Len) <= MAX_LEN_OF_SUPPORTED_RATES)
+                    {
+                        memcpy(&Rates[*RatesLen], eid_ptr->Octet, eid_ptr->Len);
+                        *RatesLen = (*RatesLen) + eid_ptr->Len;
+                    }
+                    else
+                    {
+                        memcpy(&Rates[*RatesLen], eid_ptr->Octet, MAX_LEN_OF_SUPPORTED_RATES - (*RatesLen));
+                        *RatesLen = MAX_LEN_OF_SUPPORTED_RATES;
+                    }
+                    break;
+                default:
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerAssocRspSanity - ignore unrecognized EID = %d\n", eid_ptr->Eid);
+                    break;
             }
-            *RatesLen = i;
+
+            eid_ptr = (PBEACON_EID_STRUCT)((UCHAR*)eid_ptr + 2 + eid_ptr->Len);
         }
     }
 
@@ -264,10 +262,13 @@ BOOLEAN PeerAssocParmSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN PeerDisassocSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr2,
@@ -276,7 +277,7 @@ BOOLEAN PeerDisassocSanity(
     MACFRAME *Fr = (MACFRAME *)Msg;
 
     COPY_MAC_ADDR(Addr2, &Fr->Hdr.Addr2);
-    memcpy(Reason, &Fr->Octet[0], 2);
+	*Reason = le16_to_cpup((PUSHORT)(&Fr->Octet[0]));
 
     return TRUE;
 }
@@ -287,10 +288,13 @@ BOOLEAN PeerDisassocSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN PeerDeauthSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr2,
@@ -299,7 +303,7 @@ BOOLEAN PeerDeauthSanity(
     MACFRAME *Fr = (MACFRAME *)Msg;
 
     COPY_MAC_ADDR(Addr2, &Fr->Hdr.Addr2);
-    memcpy(Reason, &Fr->Octet[0], 2);
+	*Reason = le16_to_cpup((PUSHORT)(&Fr->Octet[0]));
 
     return TRUE;
 }
@@ -310,10 +314,13 @@ BOOLEAN PeerDeauthSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN PeerAuthSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr,
@@ -325,11 +332,11 @@ BOOLEAN PeerAuthSanity(
     MACFRAME     *Fr = (MACFRAME *)Msg;
 
     COPY_MAC_ADDR(Addr,    &Fr->Hdr.Addr2);
-    memcpy(Alg,    &Fr->Octet[0], 2);
-    memcpy(Seq,    &Fr->Octet[2], 2);
-    memcpy(Status, &Fr->Octet[4], 2);
+	*Alg = le16_to_cpup((PUSHORT)(&Fr->Octet[0]));
+	*Seq = le16_to_cpup((PUSHORT)(&Fr->Octet[2]));
+	*Status = le16_to_cpup((PUSHORT)(&Fr->Octet[4]));
 
-    if (*Alg == AUTH_MODE_OPEN)
+    if (*Alg == Ndis802_11AuthModeOpen)
     {
         if (*Seq == 1 || *Seq == 2)
         {
@@ -341,7 +348,7 @@ BOOLEAN PeerAuthSanity(
             return FALSE;
         }
     }
-    else if (*Alg == AUTH_MODE_KEY)
+    else if (*Alg == Ndis802_11AuthModeShared)
     {
         if (*Seq == 1 || *Seq == 4)
         {
@@ -371,27 +378,31 @@ BOOLEAN PeerAuthSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN PeerProbeReqSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr2,
     OUT CHAR Ssid[],
-    OUT UCHAR *SsidLen,
-    OUT UCHAR Rates[],
-    OUT UCHAR *RatesLen)
+    OUT UCHAR *SsidLen)
+//    OUT UCHAR Rates[],
+//    OUT UCHAR *RatesLen)
 {
     UCHAR Idx;
-    CHAR          IeType;
+	UCHAR	RateLen;
+	CHAR          IeType;
     MACFRAME *Fr = (MACFRAME *)Msg;
 
     COPY_MAC_ADDR(Addr2, &Fr->Hdr.Addr2);
 
-    if (Fr->Octet[0] != IE_SSID || Fr->Octet[1] > MAX_LEN_OF_SSID)
+    if ((Fr->Octet[0] != IE_SSID) || (Fr->Octet[1] > MAX_LEN_OF_SSID))
     {
-        DBGPRINT(RT_DEBUG_TRACE, "PeerProbeReqSanity fail - wrong SSID IE\n");
+        DBGPRINT(RT_DEBUG_TRACE, "PeerProbeReqSanity fail - wrong SSID IE(Type=%d,Len=%d)\n",Fr->Octet[0],Fr->Octet[1]);
         return FALSE;
     }
 
@@ -400,29 +411,18 @@ BOOLEAN PeerProbeReqSanity(
 
     Idx = *SsidLen + 2;
 
-    // -- get supported rates from payload and advance the pointer
-    IeType = Fr->Octet[Idx];
-    *RatesLen = Fr->Octet[Idx + 1];
-    if (IeType != IE_SUPP_RATES)
+	// -- get supported rates from payload and advance the pointer
+	IeType = Fr->Octet[Idx];
+	RateLen = Fr->Octet[Idx + 1];
+	if (IeType != IE_SUPP_RATES)
     {
-    	DBGPRINT(RT_DEBUG_TRACE, "PeerAssocParmSanity fail - wrong SupportedRates IE\n");
-    	return FALSE;
+        DBGPRINT(RT_DEBUG_TRACE, "PeerProbeReqSanity fail - wrong SupportRates IE(Type=%d,Len=%d)\n",Fr->Octet[Idx],Fr->Octet[Idx+1]);
+        return FALSE;
     }
-    else
-    {
-    	int 	index;
-    	UCHAR	tmp_rate, i;
-
-    	i = 0;
-    	Idx += 2;
-    	for (index = 0; index < *RatesLen; index++)
-    	{
-    		// Mask out basic rate set bit
-    		tmp_rate = Fr->Octet[Idx + index] & 0x7f;
-    		if ((tmp_rate == 0x02) || (tmp_rate == 0x04) || (tmp_rate == 0x0B) || (tmp_rate == 0x16))
-    			Rates[i++] = Fr->Octet[Idx + index];	// Save rate with basic rate set bit if exists
-    	}
-    	*RatesLen = i;
+	else
+	{
+		if ((pAd->PortCfg.AdhocMode == 2) && (RateLen < 8))
+			return (FALSE);
 	}
 
     return TRUE;
@@ -434,10 +434,13 @@ BOOLEAN PeerProbeReqSanity(
         MLME message sanity check
     Return:
         TRUE if all parameters are OK, FALSE otherwise
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
-BOOLEAN BeaconAndProbeRspSanity(
-    IN PRTMP_ADAPTER pAd,
+BOOLEAN PeerBeaconAndProbeRspSanity(
+    IN PRT2570ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
     OUT MACADDR *Addr2,
@@ -454,16 +457,32 @@ BOOLEAN BeaconAndProbeRspSanity(
     OUT USHORT *CapabilityInfo,
     OUT UCHAR Rate[],
     OUT UCHAR *RateLen,
+    OUT BOOLEAN *ExtendedRateIeExist,
+    OUT UCHAR *Erp,
     OUT UCHAR *DtimCount,
     OUT UCHAR *DtimPeriod,
     OUT UCHAR *BcastFlag,
     OUT UCHAR *MessageToMe,
-    OUT UCHAR *Legacy)
+    OUT UCHAR *Legacy,
+    OUT UCHAR SupRate[],
+	OUT UCHAR *SupRateLen,
+	OUT UCHAR ExtRate[],
+	OUT UCHAR *ExtRateLen,
+    OUT UCHAR *LengthVIE,
+    OUT	PNDIS_802_11_VARIABLE_IEs pVIE)
 {
     CHAR				*Ptr, TimLen;
     MACFRAME			*Fr;
     PBEACON_EID_STRUCT	eid_ptr;
-    UCHAR				SubType,CheckIdx=0;
+    UCHAR				SubType;
+	UCHAR				Sanity;
+
+	// Add for 3 necessary EID field check
+	Sanity = 0;
+
+    *ExtendedRateIeExist = FALSE;
+    *Erp = 0;
+	*LengthVIE = 0;					// Set the length of VIE to init value 0
 
     Fr = (MACFRAME *)Msg;
 
@@ -481,11 +500,11 @@ BOOLEAN BeaconAndProbeRspSanity(
     Ptr += TIMESTAMP_LEN;
 
     // get beacon interval from payload and advance the pointer
-    memcpy(BeaconPeriod, Ptr, 2);
+	*BeaconPeriod = le16_to_cpup((PUSHORT)(Ptr));
     Ptr += 2;
 
     // get capability info from payload and advance the pointer
-    memcpy(CapabilityInfo, Ptr, 2);
+	*CapabilityInfo = le16_to_cpup((PUSHORT)(Ptr));
     Ptr += 2;
     if (CAP_IS_ESS_ON(*CapabilityInfo))
     {
@@ -496,6 +515,9 @@ BOOLEAN BeaconAndProbeRspSanity(
         *BssType = BSS_INDEP;
     }
 
+	// Mask out unnecessary capability information
+	*CapabilityInfo &= SUPPORTED_CAPABILITY_INFO;
+
     eid_ptr = (PBEACON_EID_STRUCT) Ptr;
 
     // get variable fields from payload and advance the pointer
@@ -504,72 +526,76 @@ BOOLEAN BeaconAndProbeRspSanity(
         switch(eid_ptr->Eid)
         {
             case IE_SSID:
+				// Already has one SSID EID in this beacon, ignore the second one
+				if (Sanity & 0x1)
+					break;
                 if(eid_ptr->Len <= MAX_LEN_OF_SSID)
                 {
-                    CheckIdx++;
                     memcpy(Ssid, eid_ptr->Octet, eid_ptr->Len);
                     *SsidLen = eid_ptr->Len;
+                	if(eid_ptr->Len < MAX_LEN_OF_SSID) {
+						Ssid[eid_ptr->Len] = 0;
+					}
+					Sanity |= 0x1;
                 }
                 else
                 {
-                    DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_SSID) \n");
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_SSID (len=%d)\n",eid_ptr->Len);
                     return FALSE;
                 }
                 break;
 
             case IE_SUPP_RATES:
-            	// Add special care for b / g mixed mode AP which will send with more than 8 supported rates.
-            	// We have to parse this rate one by one and make sure its 802.11 b supported.
-            	{
+                if(eid_ptr->Len <= MAX_LEN_OF_SUPPORTED_RATES)
+                {
             		int		index;
-            		UCHAR	tmp_rate, i;
+            		UCHAR	rate, i;
             		PUCHAR	eid_rate;
 
             		i = 0;
-            		CheckIdx++;
             		eid_rate = eid_ptr->Octet;
             		for (index = 0; index < eid_ptr->Len; index++)
             		{
-            			// Mask out basic rate set bit
-            			tmp_rate = eid_rate[index] & 0x7f;
-            			if ((tmp_rate == 0x02) || (tmp_rate == 0x04) || (tmp_rate == 0x0B) || (tmp_rate == 0x16))
+            			rate = eid_rate[index] & 0x7f; // Mask out basic rate set bit
+            			if ((rate == 2) || (rate == 4) || (rate == 11) || (rate == 22) ||
+            			    (rate == 12) || (rate == 18) || (rate == 24) || (rate == 36) ||
+            			    (rate == 48) || (rate == 72) || (rate == 96) || (rate == 108))
             				Rate[i++] = eid_rate[index];	// Save rate with basic rate set bit if exists
             		}
             		*RateLen = i;
-            	}
-            	/*
-                if(eid_ptr->Len <= MAX_LEN_OF_SUPPORTED_RATES)
-                {
-                    CheckIdx++;
-                    memcpy(Rate, eid_ptr->Octet, eid_ptr->Len);
-                    *RateLen = eid_ptr->Len;
+					Sanity |= 0x2;
+					// Copy supported rate from desired AP's beacon. We are trying to match
+					// AP's supported and extended rate settings.
+					memcpy(SupRate, eid_ptr->Octet, eid_ptr->Len);
+					*SupRateLen = eid_ptr->Len;
+					// Check against the supported rates
+					RTMPCheckRates(pAd, SupRate, SupRateLen);
                 }
                 else
                 {
-                    DBGPRINT(RT_DEBUG_TRACE, ("BeaconAndProbeRspSanity(IE_SUPP_RATES) \n"));
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_SUPP_RATES (len=%d)\n",eid_ptr->Len);
                     return FALSE;
                 }
-                */
                 break;
 
             case IE_FH_PARM:
-                DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_FH_PARM) \n");
+                DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity(IE_FH_PARM) \n");
                 break;
 
             case IE_DS_PARM:
                 if(eid_ptr->Len == 1)
                 {
-                    CheckIdx++;
                     *Channel = *eid_ptr->Octet;
                     if (ChannelSanity(pAd, *Channel) == 0)
                     {
-                    	DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_DS_PARM)\n");
-                    	return FALSE;
+                        DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_DS_PARM (ch=%d)\n",*Channel);
+                        return FALSE;
                     }
+					Sanity |= 0x4;
                 }
                 else
                 {
-                    DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_DS_PARM)\n");
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_DS_PARM (len=%d)\n",eid_ptr->Len);
                     return FALSE;
                 }
                 break;
@@ -578,11 +604,15 @@ BOOLEAN BeaconAndProbeRspSanity(
                 if(eid_ptr->Len == 6)
                 {
                     *CfExist = TRUE;
-                    memcpy(CfParm, eid_ptr->Octet, eid_ptr->Len);
+                    //memcpy(CfParm, eid_ptr->Octet, eid_ptr->Len);
+    				CfParm->CfpCount = eid_ptr->Octet[0];
+    				CfParm->CfpPeriod = eid_ptr->Octet[1];
+    				CfParm->CfpMaxDuration = le16_to_cpup((PUSHORT)&eid_ptr->Octet[2]);
+    				CfParm->CfpDurRemaining = le16_to_cpup((PUSHORT)&eid_ptr->Octet[4]);
                 }
                 else
                 {
-                    DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_CF_PARM)\n");
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_CF_PARM\n");
                     return FALSE;
                 }
                 break;
@@ -590,11 +620,11 @@ BOOLEAN BeaconAndProbeRspSanity(
             case IE_IBSS_PARM:
                 if(eid_ptr->Len == 2)
                 {
-                    memcpy(AtimWin, eid_ptr->Octet, eid_ptr->Len);
+					*AtimWin = le16_to_cpup((PUSHORT)eid_ptr->Octet);
                 }
                 else
                 {
-                    DBGPRINT(RT_DEBUG_TRACE, "BeaconAndProbeRspSanity(IE_IBSS_PARM)\n");
+                    DBGPRINT(RT_DEBUG_TRACE, "PeerBeaconAndProbeRspSanity - wrong IE_IBSS_PARM\n");
                     return FALSE;
                 }
                 break;
@@ -604,19 +634,100 @@ BOOLEAN BeaconAndProbeRspSanity(
                 {
                     GetTimBit((PUCHAR)eid_ptr, pAd->PortCfg.Aid, &TimLen, BcastFlag, DtimCount, DtimPeriod, MessageToMe);
                 }
+                break;
+
+			// New for WPA
+			case IE_WPA:
+				// Check the OUI version, filter out non-standard usage
+				if ((memcmp(eid_ptr->Octet, WPA_OUI, 4) == 0))
+				{
+					// Copy to pVIE which will report to microsoft bssid list.
+					Ptr = (PUCHAR) pVIE;
+					memcpy(Ptr + *LengthVIE, &eid_ptr->Eid, eid_ptr->Len + 2);
+					*LengthVIE += (eid_ptr->Len + 2);
+				}
+				DBGPRINT(RT_DEBUG_INFO, "PeerBeaconAndProbeRspSanity - Receive IE_WPA\n");
+				break;
+
+			case IE_EXT_SUPP_RATES:
+			    // concatenate all extended rates to Rates[] and RateLen
+			    *ExtendedRateIeExist = TRUE;
+                if (eid_ptr->Len <= MAX_LEN_OF_SUPPORTED_RATES)
+                {
+            		int		index;
+            		UCHAR	rate, i;
+            		PUCHAR	eid_rate;
+
+            		i = *RateLen;
+            		eid_rate = eid_ptr->Octet;
+            		for (index = 0; index < eid_ptr->Len; index++)
+            		{
+            			rate = eid_rate[index] & 0x7f; // Mask out basic rate set bit
+            			if ((rate == 2) || (rate == 4) || (rate == 11) || (rate == 22) ||
+            			    (rate == 12) || (rate == 18) || (rate == 24) || (rate == 36) ||
+            			    (rate == 48) || (rate == 72) || (rate == 96) || (rate == 108))
+            				Rate[i++] = eid_rate[index];	// Save rate with basic rate set bit if exists
+
+            		    if (i >= MAX_LEN_OF_SUPPORTED_RATES)
+            		        break;
+            		}
+            		*RateLen = i;
+					// Copy extended rate from desired AP's beacon. We are trying to match
+					// AP's supported and extended rate settings.
+					memcpy(ExtRate, eid_ptr->Octet, eid_ptr->Len);
+					*ExtRateLen = eid_ptr->Len;
+					// Check against the supported rates
+					RTMPCheckRates(pAd, ExtRate, ExtRateLen);
+				}
+			    break;
+
+            case IE_ERP:
+                if (eid_ptr->Len == 1)
+                {
+                    *Erp = (UCHAR)eid_ptr->Octet[0];
+                }
+                break;
+
+			// WPA2 & 802.11i RSN
+			case IE_RSN:
+				// There is no OUI for version anymore, check the group cipher OUI before copying
+				if ((memcmp(eid_ptr->Octet + 2, RSN_OUI, 3) == 0))
+				{
+					// Copy to pVIE which will report to microsoft bssid list.
+					Ptr = (PUCHAR) pVIE;
+					memcpy(Ptr + *LengthVIE, &eid_ptr->Eid, eid_ptr->Len + 2);
+					*LengthVIE += (eid_ptr->Len + 2);
+				}
+                DBGPRINT(RT_DEBUG_INFO, "IE_RSN length = %d\n", eid_ptr->Len);
                 break;
 
             default:
-                DBGPRINT(RT_DEBUG_INFO, "BeaconAndProbeRspSanity Unrecognized EID = %d in Beacon or ProbeRsp\n", eid_ptr->Eid);
+                DBGPRINT(RT_DEBUG_INFO,
+						"- %s - unrecognized EID = %d Len = %d:",
+						__FUNCTION__, eid_ptr->Eid, eid_ptr->Len);
+				{
+					int i;
+					for (i = 0; i < eid_ptr->Len; i++) {
+						DBGPRINT_RAW(RT_DEBUG_INFO, " %02x", eid_ptr->Octet[i]);
+					}
+					DBGPRINT_RAW(RT_DEBUG_INFO, "\n");
+				}
                 break;
         }
 
         eid_ptr = (PBEACON_EID_STRUCT)((UCHAR*)eid_ptr + 2 + eid_ptr->Len);
     }
 
-    if(CheckIdx < 3)
+    // in 802.11a band, AP may skip this DS IE in their BEACON
+    if ((pAd->PortCfg.Channel > 14) && ((Sanity & 0x04)==0))
     {
-        DBGPRINT(RT_DEBUG_WARN, "BeaconAndProbeRspSanity(Check mandatory field failed)\n");
+        *Channel = pAd->PortCfg.Channel;
+        Sanity |= 0x04;
+    }
+
+    if (Sanity != 0x7)
+    {
+        DBGPRINT(RT_DEBUG_WARN, "PeerBeaconAndProbeRspSanity - missing field, Sanity=0x%02x\n", Sanity);
         return FALSE;
     }
     else
@@ -629,6 +740,9 @@ BOOLEAN BeaconAndProbeRspSanity(
 /*
     ==========================================================================
     Description:
+
+	IRQL = DISPATCH_LEVEL
+
     ==========================================================================
  */
 BOOLEAN GetTimBit(
@@ -665,7 +779,6 @@ BOOLEAN GetTimBit(
     else
         *BcastFlag = FALSE;
 
-#if 1
     // Parse Partial Virtual Bitmap from TIM element
     N1 = BitCntl & 0xfe;    // N1 is the first bitmap byte#
     N2 = *TimLen - 4 + N1;  // N2 is the last bitmap byte#
@@ -687,12 +800,10 @@ BOOLEAN GetTimBit(
         else
             *MessageToMe = FALSE;
     }
-#endif
 
     return TRUE;
 }
 
-
 /*!
  *  \brief Get legacy bit, right now for 11b it is always 0
  *  \param
@@ -708,54 +819,152 @@ BOOLEAN GetLegacy(
     return TRUE;
 }
 
+// IRQL = DISPATCH_LEVEL
 UCHAR ChannelSanity(
-    IN PRTMP_ADAPTER pAd,
+    IN PRT2570ADAPTER pAd,
     IN UCHAR channel)
 {
-	switch (pAd->PortCfg.CountryRegion)
-	{
-		case REGION_FCC:	// 1 - 11
-			if ((channel > 0) && (channel < 12))
-				return 1;
-			break;
+    UCHAR index;
 
-		case REGION_IC:		// 1 -11
-			if ((channel > 0) && (channel < 12))
-				return 1;
-			break;
-
-		case REGION_ETSI:	// 1 - 13
-			if ((channel > 0) && (channel < 14))
-				return 1;
-			break;
-
-		case REGION_SPAIN:	// 10 - 11
-			if ((channel > 9) && (channel < 12))
-				return 1;
-			break;
-
-		case REGION_FRANCE:	// 10 -13
-			if ((channel > 9) && (channel < 14))
-				return 1;
-			break;
-
-		case REGION_MKK:	// 14
-			if (channel == 14)
-				return 1;
-			break;
-
-		case REGION_MKK1:	// 1 - 14
-			if ((channel > 0) && (channel < 15))
-				return 1;
-			break;
-
-		case REGION_ISRAEL:	// 3 - 9
-			if ((channel > 2) && (channel < 10))
-				return 1;
-			break;
-
-		default:			// Error
-			return 0;
-	}
-    return (0);
+    for (index = 0; index < pAd->PortCfg.ChannelListNum; index ++)
+    {
+        if (channel == pAd->PortCfg.ChannelList[index])
+            return 1;
+    }
+    return 0;
 }
+
+/*
+	========================================================================
+	Routine Description:
+		Sanity check NetworkType (11b, 11g or 11a)
+
+	Arguments:
+		pBss - Pointer to BSS table.
+
+	Return Value:
+        Ndis802_11DS .......(11b)
+        Ndis802_11OFDM24....(11g)
+        Ndis802_11OFDM5.....(11a)
+
+	IRQL = DISPATCH_LEVEL
+
+	========================================================================
+*/
+NDIS_802_11_NETWORK_TYPE NetworkTypeInUseSanity(
+    IN PBSS_ENTRY pBss)
+{
+	NDIS_802_11_NETWORK_TYPE	NetWorkType = Ndis802_11DS;
+	UCHAR						rate, i;
+
+	if (pBss->Channel <= 14)
+	{
+		for (i = 0; i < pBss->RatesLen; i++)
+		{
+			rate = pBss->Rates[i] & 0x7f; // Mask out basic rate set bit
+			if ((rate == 2) || (rate == 4) || (rate == 11) || (rate == 22))
+			{
+				//
+				// basic rate means Ndis802_11DS
+				//
+				NetWorkType = Ndis802_11DS;
+				continue;
+			}
+			else
+			{
+				//
+				// Otherwise (even rate > 108) means Ndis802_11OFDM24
+				//
+				NetWorkType = Ndis802_11OFDM24;
+				break;
+			}
+		}
+	}
+	else
+	{
+		NetWorkType = Ndis802_11OFDM5;
+	}
+
+	return NetWorkType;
+}
+
+/*
+	========================================================================
+	Routine Description:
+		Sanity check pairwise key on pAdapter->PortCfg.AuthMode < Ndis802_11AuthModeWPA
+
+	Arguments:
+		pAdapter - Pointer to our adapter
+		pBuf 	 - Pointer to NDIS_802_11_KEY structure
+
+	Return Value:
+		NDIS_STATUS_SUCCESS
+		NDIS_STATUS_INVALID_DATA
+
+	IRQL = DISPATCH_LEVEL
+
+	Note:
+		For OID_802_11_ADD_KEY setting, on old wep stuff also need to verify
+		the structure of NIDS_802_11_KEY
+	========================================================================
+*/
+NDIS_STATUS	RTMPWPAWepKeySanity(
+	IN	PRT2570ADAPTER	pAdapter,
+	IN	PVOID			pBuf)
+{
+	PNDIS_802_11_KEY	pKey;
+	ULONG				KeyIdx;
+	NDIS_STATUS			Status = NDIS_STATUS_SUCCESS;
+	BOOLEAN 			bTxKey; 		// Set the key as transmit key
+	BOOLEAN 			bPairwise;		// Indicate the key is pairwise key
+
+	pKey = (PNDIS_802_11_KEY) pBuf;
+	KeyIdx = pKey->KeyIndex & 0x0fffffff;
+	// Bit 31 of Add-key, Tx Key
+	bTxKey		   = (pKey->KeyIndex & 0x80000000) ? TRUE : FALSE;
+	// Bit 30 of Add-key PairwiseKey
+	bPairwise	   = (pKey->KeyIndex & 0x40000000) ? TRUE : FALSE;
+
+	// 1. Check Group / Pairwise Key
+	if (bPairwise)	// Pairwise Key
+	{
+		// 1. Check KeyIdx
+		// it is a shared key
+		if (KeyIdx > 4)
+			return (-EINVAL);
+
+		// 2. Check bTx, it must be true, otherwise, return NDIS_STATUS_INVALID_DATA
+		if (bTxKey == FALSE)
+			return (-EINVAL);
+
+		// 3. If BSSID is not all 0xff, return NDIS_STATUS_INVALID_DATA
+		if ((memcmp(pKey->BSSID, BCAST, 6) == 0))
+			return (-EINVAL);
+
+		// check key length
+		if ((pKey->KeyLength != 5) && (pKey->KeyLength != 13))
+			return (-EINVAL);
+	}
+	else
+	{
+		// Group Key
+		// 1. Check BSSID, if not current BSSID or Bcast, return NDIS_STATUS_INVALID_DATA
+		if ((memcmp(&pKey->BSSID, &BCAST, 6) != 0) &&
+			(memcmp(&pKey->BSSID, &pAdapter->PortCfg.Bssid, 6) != 0))
+			return (-EINVAL);
+		// 2. Check Key index for supported Group Key
+		if (KeyIdx > 4)
+			return (-EINVAL);
+	}
+
+	pAdapter->PortCfg.SharedKey[KeyIdx].KeyLen = (UCHAR) pKey->KeyLength;
+	memcpy(pAdapter->PortCfg.SharedKey[KeyIdx].Key, &pKey->KeyMaterial, pKey->KeyLength);
+	if (pKey->KeyIndex & 0x80000000)
+	{
+		// Default key for tx (shared key)
+		pAdapter->PortCfg.DefaultKeyId = (UCHAR) KeyIdx;
+	}
+
+	return (Status);
+}
+
